
#    MIDAS Access Server  for FEBEX 


package require MIDASDomainHandler     ;#  load default support procedures

set z [catch {package require FEBEXAccess} m]
if {$z != 0} {puts $m}


global MIDAS_FEBEXAccessServer_Exists
set MIDAS_FEBEXAccessServer_Exists 1

global MIDAS_FEBEXAccessServer_Verbose
set MIDAS_FEBEXAccessServer_Verbose 0


if {![info exists ::FEBEX::AccessTestMode]} {set ::FEBEX::AccessTestMode 0}


# Use namespaces to isolate our methods
namespace eval urn:FEBEXAccessServer {

    proc AccessServerVerbose {option} {
        do_AccessServerVerbose $option
        return [rpcvar::rpcvar int 0]
    }

    proc Write {Link Board Address Value} {
        do_Write $Link $Board $Address $Value
        return [rpcvar::rpcvar int 0]
    }

    proc Read {Link Board Address} {
        set vl [do_Read $Link $Board $Address]
        return [rpcvar::rpcvar int $vl]
    }
    
    proc SetBit {Link Board Address Value} {
        set vl [do_SetBit $Link $Board $Address $Value]
        return [rpcvar::rpcvar int $vl]
    }
    
    proc ClearBit {Link Board Address Value} {
        set vl [do_ClearBit $Link $Board $Address $Value]
        return [rpcvar::rpcvar int $vl]
    }
    
    proc Init {Link Board} {
        set vl [do_Init $Link $Board]
        return [rpcvar::rpcvar int $vl]
    }
    
    proc Device {DevNum} {
        set vl [do_Device $DevNum]
        return [rpcvar::rpcvar int $vl]
    }
    
    proc Reset {DevNum} {
        set vl [do_Reset $DevNum]
        return [rpcvar::rpcvar int $vl]
    }
    
    proc Configure {file} {
        set vl [do_Configure $file]
    }
    
    proc Verify {file} {
        set vl [do_Verify $file]
    }
    
    proc do_AccessServerVerbose {option} {
        global MIDAS_FEBEXAccessServer_Verbose
        if {$MIDAS_FEBEXAccessServer_Verbose} {puts "called method AccessServerVerbose with parameter $option"}
        set MIDAS_FEBEXAccessServer_Verbose $option
        FEBEXAccess_Verbose $option
    }

    proc do_Write {Link Board Address Value} {
       global MIDAS_FEBEXAccessServer_Verbose

       if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_Write $Link $Board $Address ([format "0x%x" $Address]) = $Value"}

       if {$::FEBEX::AccessTestMode != 0} {return "0 0x0"}

       set z [catch {FEBEXAccess_Write -l $Link -s $Board -a $Address -d $Value} m]

       if {$z == 0} then {
          return 0
       } else {
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n  $m"
       }
    }

    proc do_Read {Link Board Address} {
       global MIDAS_FEBEXAccessServer_Verbose

       if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_Read $Link $Board $Address ([format "0x%x" $Address])"}

       if {$::FEBEX::AccessTestMode != 0} {return "0 0x5aaaaa55"}

       set z [catch {FEBEXAccess_Read -l $Link -s $Board -a $Address} m]

       if {$z == 0} then {
          if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_Read $Address ([format "0x%x" $Address]) = $m"}
          if {[llength $m] == 1} {set RData $m} else {set RData [lindex $m 1]}
          return $RData
       } else {
           if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_Read rc=$z ($m) $Address ([format "0x%x" $Address])"}
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n  $m"
       }
    }
    
    proc do_SetBit {Link Board Address Value} {
       global MIDAS_FEBEXAccessServer_Verbose

       if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_SetBit $Link $Board $Address ([format "0x%x" $Address]) $Value ([format "0x%x" $Value])"}

       if {$::FEBEX::AccessTestMode != 0} {return "0 0x0"}

       set z [catch {FEBEXAccess_SetBit -l $Link -s $Board -a $Address -d $Value} m]

       if {$z == 0} then {
          if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_SetBit $Address ([format "0x%x" $Address]) = $m"}
          return $m
       } else {
           if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_SetBit rc=$z ($m) $Address ([format "0x%x" $Address])"}
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n  $m"
       }
    }
    
   proc do_ClearBit {Link Board Address Value} {
       global MIDAS_FEBEXAccessServer_Verbose

       if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_ClearBit $Link $Board $Address ([format "0x%x" $Address]) $Value ([format "0x%x" $Value])"}

       if {$::FEBEX::AccessTestMode != 0} {return "0 0x0"}

       set z [catch {FEBEXAccess_ClearBit -l $Link -s $Board -a $Address -d $Value} m]

       if {$z == 0} then {
          if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_ClearBit $Address ([format "0x%x" $Address]) = $m"}
          return $m
       } else {
           if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_ClearBit rc=$z ($m) $Address ([format "0x%x" $Address])"}
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n  $m"
       }
    }
    
   proc do_Init {Link Board} {
       global MIDAS_FEBEXAccessServer_Verbose

       if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_Init"}

       if {$::FEBEX::AccessTestMode != 0} {return "0 0x0"}

       set z [catch {FEBEXAccess_Init -l $Link -s $Board} m]

       if {$z == 0} then {
          if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_Init"}
          return $m
       } else {
           if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_Init rc=$z ($m)"}
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n  $m"
       }
    }
    
   proc do_Configure {FileName} {
       global MIDAS_FEBEXAccessServer_Verbose

       if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_Configure $FileName"}

       if {$::FEBEX::AccessTestMode != 0} {return "0 0x0"}

       set z [catch {FEBEXAccess_Configure $FileName} m]

       if {$z == 0} then {
          if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_Configure $FileName = $m"}
          return $m
       } else {
           if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_Configure rc=$z ($m)"}
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n  $m"
       }
    }
    
   proc do_Verify {FileName} {
       global MIDAS_FEBEXAccessServer_Verbose

       if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_Verify $FileName"}

       if {$::FEBEX::AccessTestMode != 0} {return "0 0x0"}

       set z [catch {FEBEXAccess_Verify $FileName} m]

       if {$z == 0} then {
          if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_Verify $FileName = $m"}
          return $m
       } else {
           if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_Verify rc=$z ($m)"}
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n  $m"
       }
    }
    
   proc do_Device {DevNum} {
       global MIDAS_FEBEXAccessServer_Verbose

       if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_Device $DevNum"}

       if {$::FEBEX::AccessTestMode != 0} {return "0 0x0"}

       set z [catch {FEBEXAccess_Device $DevNum} m]

       if {$z == 0} then {
          if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_Device $DevNum = $m"}
          return $m
       } else {
           if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_Device rc=$z ($m)"}
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n  $m"
       }
    }
    
   proc do_Reset {DevNum} {
       global MIDAS_FEBEXAccessServer_Verbose

       if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_Reset $DevNum"}

       if {$::FEBEX::AccessTestMode != 0} {return "0 0x0"}

       set z [catch {FEBEXAccess_Reset $DevNum} m options]
       
       puts "do_Reset $m $options"

       if {$z == 0} then {
          if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_Reset $DevNum = $m"}
          return $m
       } else {
           if {$MIDAS_FEBEXAccessServer_Verbose} {puts "FEBEX: do_Reset rc=$z ($m)"}
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n  $m"
       }
    }
    
    
   proc do_FEBEXReadRegisters {board registers} {
        global MIDAS_FEBEXAccessServer_Verbose

        if {$MIDAS_FEBEXAccessServer_Verbose} \
             {puts "called method do_FEBEXReadRegisters"}
             
        set link $::febexU::link
             
  if {$::FEBEX::AccessTestMode != 0} {
     global FEBEXData
     if {![info exists FEBEXData($link,$board,$registers)]} {set FEBEXData($link,$board,$registers) 0xaaaa}
     InsertLog "do_FEBEXReadRegisters $link,$board,$registers  reads  [set FEBEXData($link,$board,$registers)]"
     return [set FEBEXData($link,$board,$registers)]
  }

        set rc {}
        foreach register $registers {
            lappend rc [do_Read $link $board $register]
        }

        if {$MIDAS_FEBEXAccessServer_Verbose}  {puts "FEBEX reply $rc"}

        return $rc
    }
    
    proc do_FEBEXWriteRegisters {board registers} {
        global MIDAS_FEBEXAccessServer_Verbose

        if {$MIDAS_FEBEXAccessServer_Verbose} \
             {puts "called method do_FEBEXWriteRegisters"}
             
     set link $::febexU::link
     
  if {$::FEBEX::AccessTestMode != 0} {
     global FEBEXData   
     if {![info exists FEBEXData($link,$board,$registers)]} {set FEBEXData($link,$board,$registers) $registers}
     InsertLog "do_FEBEXWriteRegisters $link,$board,$registers  writes  [set FEBEXData($link,$board,$registers)]"
     return [set FEBEXData($link,$board,$registers)]
  }

        foreach register $registers {
            set rc [do_Write $link $board $register]
#            if {$rc != 0} {break}
        }

        if {$MIDAS_FEBEXAccessServer_Verbose}  {puts "FEBEX reply $rc"}

        return $rc
    }    




    # We have to publish the public methods...

     SOAP::export AccessServerVerbose
     SOAP::export Write
     SOAP::export Read
     SOAP::export SetBit
     SOAP::export ClearBit
     SOAP::export Device
     SOAP::export Init
     SOAP::export Reset
     SOAP::export Configure
     SOAP::export Verify
}

# register this service with tclhttpd
puts [SOAP::Domain::register  -prefix  /FEBEXAccessServer  -namespace urn:FEBEXAccessServer -uri urn:FEBEXAccessServer]


proc ::SOAP::Domain::MIDAS_SOAPDomainHandler_/FEBEXAccessServer {optsname sock args} {   \
    ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DefaultAccessServer $optsname $sock $args}

package provide FEBEXAccessServer 1.0
